@@ -1,11 +1,11 @@
 package Apache2::AuthCookie::Params;
-$Apache2::AuthCookie::Params::VERSION = '3.22';
+$Apache2::AuthCookie::Params::VERSION = '3.24';
 # ABSTRACT: AuthCookie Params Driver for mod_perl 2.x
 
 use strict;
 use warnings;
 use base 'Apache::AuthCookie::Params::Base';
-use Class::Load qw(try_load_class load_class);
+use Class::Load qw(try_load_class);
 
 sub _new_instance {
     my ($class, $r) = @_;
@@ -20,11 +20,9 @@ sub _new_instance {
         return Apache2::Request->new($r);
     }
     else {
-        load_class('CGI');
-
         $r->server->log_error("params: using CGI") if $debug >= 3;
 
-        return CGI->new($r);
+        return $class->_cgi_new($r);
     }
 
     return;
@@ -42,7 +40,7 @@ Apache2::AuthCookie::Params - AuthCookie Params Driver for mod_perl 2.x
 
 =head1 VERSION
 
-version 3.22
+version 3.24
 
 =head1 SYNOPSIS
 
